@@ -1,17 +1,19 @@
 import itertools
 import json
 import numpy as np
+from typing import List, Optional
 from shapely.affinity import translate, rotate
 from shapely.geometry import box
 
 from gdshelpers.geometry.shapely_adapter import convert_to_layout_objs, bounds_union, transform_bounds
 from gdshelpers.export.gdsii_export import write_cell_to_gdsii_file
 from gdshelpers.geometry import geometric_union
+from gdshelpers.parts.port import Port
 import gdshelpers.helpers.layers as std_layers
 
 
 class Cell:
-    def __init__(self, name):
+    def __init__(self, name: str):
         """
         Creates a new Cell named `name` at `origin`
 
@@ -23,7 +25,6 @@ class Cell:
         self.dlw_data = {}
         self.desc = {'dlw': self.dlw_data, 'desc': {}, 'ebl': []}
         self.cell_gdspy = None
-        self.cell_gdscad = None
         self.cell_oasis = None
         self._bounds = None
         # Only contains the bounds of items in `layer_dict`.
@@ -38,7 +39,7 @@ class Cell:
         """
         return self.get_bounds(layers=None)
 
-    def get_bounds(self, layers=None):
+    def get_bounds(self, layers: Optional[List[int]] = None):
         """
         Calculates and returns the envelope for the given layers.
         Returns `None` if it is empty.
@@ -51,7 +52,7 @@ class Cell:
             for layer in layers or self.layer_dict.keys():
                 for geo in self.layer_dict.get(layer, []):
                     geo_bounds = geo.get_shapely_object().bounds if hasattr(geo, 'get_shapely_object') else geo.bounds
-                    if geo_bounds is not ():  # Some shapely geometries (collections) can return empty bounds
+                    if geo_bounds != ():  # Some shapely geometries (collections) can return empty bounds
                         bounds.append(geo_bounds)
 
             # Cache envelope if we have the global envelope
@@ -77,7 +78,7 @@ class Cell:
         else:
             return bounds[2] - bounds[0], bounds[3] - bounds[1]
 
-    def add_to_layer(self, layer, *geometry):
+    def add_to_layer(self, layer: int, *geometry):
         """
         Adds a shapely geometry to a the layer
 
@@ -103,7 +104,7 @@ class Cell:
             raise ValueError('ID "{:s}" already used'.format(dlw_id))
         self.dlw_data[dlw_type][dlw_id] = data
 
-    def add_cell(self, cell, origin=(0, 0), angle=None, columns=1, rows=1, spacing=None):
+    def add_cell(self, cell, origin=(0, 0), angle: Optional[float] = None, columns=1, rows=1, spacing=None):
         """
         Adds a Cell to this cell
 
@@ -123,14 +124,14 @@ class Cell:
             dict(cell=cell, origin=origin, angle=angle, magnification=None, x_reflection=False, columns=columns,
                  rows=rows, spacing=spacing))
 
-    def add_region_layer(self, region_layer=std_layers.regionlayer, layers=None):
+    def add_region_layer(self, region_layer: int = std_layers.regionlayer, layers: Optional[List[int]] = None):
         """
         Generate a region layer around all objects on `layers` and place it on layer `region_layer`.
         If `layers` is None, all layers are used.
         """
         self.add_to_layer(region_layer, box(*self.get_bounds(layers)))
 
-    def add_frame(self, padding=30., line_width=1., frame_layer=std_layers.framelayer, bounds=None):
+    def add_frame(self, padding=30., line_width=1., frame_layer: int = std_layers.framelayer, bounds=None):
         """
         Generates a rectangular frame around the contents of the cell.
 
@@ -147,7 +148,7 @@ class Cell:
         frame = frame.difference(frame.buffer(-line_width))
         self.add_to_layer(frame_layer, frame)
 
-    def add_ebl_marker(self, layer, marker):
+    def add_ebl_marker(self, layer: int, marker):
         """
         Adds an Marker to the layout
 
@@ -157,7 +158,7 @@ class Cell:
         self.add_to_layer(layer, marker)
         self.desc['ebl'].append(list(marker.origin))
 
-    def add_ebl_frame(self, layer, frame_generator, bounds=None, **kwargs):
+    def add_ebl_frame(self, layer: int, frame_generator, bounds=None, **kwargs):
         """
         Adds global markers to the layout
 
@@ -248,32 +249,6 @@ class Cell:
                         self.cell_gdspy.add(convert_to_layout_objs(geometry, layer, library='gdspy'))
         return self.cell_gdspy
 
-    def get_gdscad_cell(self, executor=None):
-        import gdsCAD
-        if self.cell_gdscad is None:
-            self.cell_gdscad = gdsCAD.core.Cell(self.name)
-            for sub_cell in self.cells:
-                angle = np.rad2deg(sub_cell['angle']) if sub_cell['angle'] is not None else None
-                if sub_cell['columns'] == 1 and sub_cell['rows'] == 1 and not sub_cell['spacing']:
-                    self.cell_gdscad.add(
-                        gdsCAD.core.CellReference(sub_cell['cell'].get_gdscad_cell(executor), origin=sub_cell['origin'],
-                                                  rotation=angle, magnification=sub_cell['magnification'],
-                                                  x_reflection=sub_cell['x_reflection']))
-                else:
-                    self.cell_gdscad.add(
-                        gdsCAD.core.CellArray(sub_cell['cell'].get_gdscad_cell(executor), origin=sub_cell['origin'],
-                                              rotation=angle, magnification=sub_cell['magnification'],
-                                              x_reflection=sub_cell['x_reflection'], cols=sub_cell['columns'],
-                                              rows=sub_cell['rows'], spacing=sub_cell['spacing']))
-            for layer, geometries in self.layer_dict.items():
-                for geometry in geometries:
-                    if executor:
-                        executor.submit(convert_to_layout_objs, geometry, layer, library='gdscad') \
-                            .add_done_callback(lambda future: self.cell_gdscad.add(future.result()))
-                    else:
-                        self.cell_gdscad.add(convert_to_layout_objs(geometry, layer, library='gdscad'))
-        return self.cell_gdscad
-
     def get_oasis_cells(self, grid_steps_per_micron=1000, executor=None):
         import fatamorgana
         import fatamorgana.records
@@ -347,7 +322,7 @@ class Cell:
             library = library or 'fatamorgana'
         elif name.endswith('.dxf'):
             name = name[:-4]
-            library = library or 'ezwriter'
+            library = library or 'ezdxf'
 
         library = library or 'gdshelpers'
 
@@ -379,16 +354,6 @@ class Cell:
                 binary_cells = map(gdspy.Cell.to_gds, gdspy_cells, [grid_steps_per_micron] * len(gdspy_cells))
 
             self.get_gdspy_lib().write_gds(name + '.gds', cells=[], binary_cells=binary_cells)
-        elif library == 'gdscad':
-            import gdsCAD
-            layout = gdsCAD.core.Layout(precision=1e-6 / grid_steps_per_micron)
-            if parallel:
-                from concurrent.futures import ProcessPoolExecutor
-                with ProcessPoolExecutor() as pool:
-                    layout.add(self.get_gdscad_cell(pool))
-            else:
-                layout.add(self.get_gdscad_cell())
-            layout.save(name + '.gds')
         elif library == 'fatamorgana':
             import fatamorgana
             layout = fatamorgana.OasisLayout(grid_steps_per_micron)
@@ -424,17 +389,17 @@ class Cell:
                 layout.write(f)
         elif library == 'ezdxf':
             from gdshelpers.export.dxf_export import write_cell_to_dxf_file
-            with open(name + '.gds', 'wb') as f:
+            with open(name + '.dxf', 'w') as f:
                 write_cell_to_dxf_file(f, self, grid_steps_per_micron, parallel=parallel)
         else:
-            raise ValueError('library must be either "gdscad", "gdspy" or "fatamorgana"')
+            raise ValueError('library must be either "gdshelpers", "gdspy", "fatamorgana" or "ezdxf"')
 
         dlw_data = self.get_dlw_data()
         if dlw_data:
             with open(name + '.dlw', 'w') as f:
                 json.dump(dlw_data, f, indent=True)
 
-    def save_desc(self, filename):
+    def save_desc(self, filename: str):
         """
         Saves a description file for the layout. The file format is not final yet and might change in a future release.
 
@@ -445,7 +410,7 @@ class Cell:
         with open(filename + '.desc', 'w') as f:
             json.dump(self.get_desc(), f, indent=True)
 
-    def get_reduced_layer(self, layer):
+    def get_reduced_layer(self, layer: int):
         """
         Returns a single shapely object containing the structures on a certain layer from this cell and all added cells.
 
@@ -463,7 +428,7 @@ class Cell:
             [translate_and_rotate(cell['cell'].get_reduced_layer(layer), cell['origin'], cell['angle'])
              for cell in self.cells])
 
-    def export_mesh(self, filename, layer_defs):
+    def export_mesh(self, filename: str, layer_defs):
         """
         Saves the current geometry as a mesh-file.
 
@@ -480,7 +445,7 @@ class Cell:
                                     for geometry in (lambda x: x if hasattr(x, '__iter__') else [x, ])(
             self.get_reduced_layer(layer)))).export(filename)
 
-    def get_patches(self, origin=(0, 0), angle_sum=0, angle=0, layers=None):
+    def get_patches(self, origin=(0, 0), angle_sum=0, angle=0, layers: Optional[List[int]] = None):
         from descartes import PolygonPatch
 
         def rotate_pos(pos, rotation_angle):
@@ -509,23 +474,26 @@ class Cell:
 
         return own_patches + sub_cells_patches
 
-    def save_image(self, filename, layers=None, antialiased=True, resolution=1., ylim=(None, None), xlim=(None, None)):
+    def save_image(self, filename: str, layers: Optional[List[int]] = None, antialiased=True, resolution=1.,
+                   ylim=(None, None), xlim=(None, None), scale=1.):
         """
            Save cell object as an image.
 
            You can either use a rasterized file format such as png but also formats such as SVG or PDF.
 
            :param filename: Name of the image file.
+           :param layers: Layers to show im the image
            :param resolution: Rasterization resolution in GDSII units.
            :param antialiased: Whether to use a anti-aliasing or not.
            :param ylim: Tuple of (min_x, max_x) to export.
            :param xlim: Tuple of (min_y, max_y) to export.
+           :param scale: Defines the scale of the image
            """
         import matplotlib.pyplot as plt
 
         # For vector graphics, map 1um to {resolution} mm instead of inch.
         is_vector = filename.split('.')[-1] in ('svg', 'svgz', 'eps', 'ps', 'emf', 'pdf')
-        scale = 5 / 127. if is_vector else 1.
+        scale *= 5 / 127. if is_vector else 1.
 
         fig, ax = plt.subplots()
         for patch in self.get_patches(layers=layers):
@@ -546,7 +514,7 @@ class Cell:
         plt.savefig(filename, transparent=True, bbox_inches='tight', dpi=1 / resolution)
         plt.close()
 
-    def show(self, layers=None, padding=5):
+    def show(self, layers: Optional[List[int]] = None, padding=5):
         """
         Shows the current cell
 
@@ -565,7 +533,7 @@ class Cell:
         ax.set_aspect(1)
         fig.show()
 
-    def add_dlw_marker(self, label, layer, origin):
+    def add_dlw_marker(self, label: str, layer: int, origin):
         """
         Adds a marker for 3D-hybrid integration
 
@@ -579,9 +547,10 @@ class Cell:
         self.add_to_layer(layer, DLWMarker(origin))
         self.add_to_layer(std_layers.parnamelayer1, Text(origin, 2, label, alignment='center-center'))
 
-        self.add_dlw_data('marker', label, {'origin': origin.tolist()})
+        self.add_dlw_data('marker', label, {'origin': list(origin)})
 
-    def add_dlw_taper_at_port(self, label, layer, port, taper_length, tip_width=.01, with_markers=True):
+    def add_dlw_taper_at_port(self, label: str, layer: int, port: Port, taper_length: float, tip_width=.01,
+                              with_markers=True):
         """
         Adds a taper for 3D-hybrid-integration at a certain port
 
@@ -617,7 +586,7 @@ if __name__ == '__main__':
     from gdshelpers.parts.waveguide import Waveguide
 
     # Create a cell-like object that offers a save output command '.save' which creates the .gds or .oas file by using
-    # gdsCAD,gdspy or fatamorgana
+    # gdspy or fatamorgana
     device_cell = Cell('my_cell')
     # Create a port to connect waveguide structures to
     start_port = Port(origin=(0, 0), width=1, angle=0)
@@ -631,7 +600,7 @@ if __name__ == '__main__':
     device_cell.add_to_layer(1, waveguide)
     device_cell.show()
     device_cell.save_image('chip.pdf')
-    # Creates the output file by using gdspy, gdscad or fatamorgana. To use the implemented parallel processing, set
+    # Creates the output file by using gdspy or fatamorgana. To use the implemented parallel processing, set
     # parallel=True.
     device_cell.save(name='my_design', parallel=True)
     device_cell.export_mesh('my_design.stl', layer_defs={1: (0, 1)})
