@@ -1,5 +1,3 @@
-from __future__ import print_function, division
-
 import collections
 
 import numpy as np
@@ -16,7 +14,7 @@ from gdshelpers.helpers.bezier import CubicBezierCurve
 from gdshelpers.helpers.positive_resist import convert_to_positive_resist
 
 
-class Waveguide(object):
+class Waveguide:
     def __init__(self, origin, angle, width):
         self._current_port = Port(origin, angle, width)
         self._in_port = self._current_port.inverted_direction.copy()
@@ -122,7 +120,7 @@ class Waveguide(object):
 
         self._current_port.set_port_properties(**kwargs)
         sample_points = max(int(abs(angle) / (np.pi / 2) * n_points), 2)
-        final_width = final_width or self.width
+        final_width = final_width if final_width is not None else self.width
 
         angle = normalize_phase(angle, zero_to_two_pi=True) - (0 if angle > 0 else 2 * np.pi)
 
@@ -156,9 +154,10 @@ class Waveguide(object):
         approach might be wasteful for paths like (x**2, y). You can suppress resampling for length by passing zero or
         none as sample_distance parameter.
 
-        The width of the generated waveguide may either be constant when passing a number or also be a callable
-        function, using the same parameter as the path. For generating slot/coplanar/... waveguides it is also possible
-        to pass an array of the form `[rail_width_1, gap_width_1, rail_width_2, ...]` which defines the width of each
+        The width of the generated waveguide may be constant when passing a number, or variable along the path
+        when passing an array or a callable function, using the same parameter as the path.
+        For generating slot/coplanar/... waveguides it is also possible to pass an array of the form
+        `[rail_width_1, gap_width_1, rail_width_2, ...]` which defines the width of each
         rail and the gaps between the rails. This array is also allowed to end with a gap_width for positioning the
         rails asymmetrically to the path which can be useful e.g. for strip-to-slot mode converters.
 
@@ -202,7 +201,7 @@ class Waveguide(object):
 
                 lengths = np.linspace(presample_coordinates_d1__cum_norm[0],
                                       presample_coordinates_d1__cum_norm[-1],
-                                      presample_coordinates_d1__cum_norm[-1] // sample_distance)
+                                      int(presample_coordinates_d1__cum_norm[-1] / sample_distance))
 
                 # First get the spline representation. This is needed since we manipulate these directly for roots
                 # finding.
@@ -232,15 +231,18 @@ class Waveguide(object):
         sample_coordinates = self._current_port.origin + np.einsum('ij,kj->ki', rotation_matrix, sample_coordinates)
 
         # Calculate the derivative
-        if path_derivative:
-            assert callable(path_derivative), 'The derivative of the path function must be callable'
+        if callable(path_derivative):
             if path_function_supports_numpy:
                 sample_coordinates_d1 = np.array(path_derivative(sample_t)).T
             else:
                 sample_coordinates_d1 = np.array([path_derivative(x) for x in sample_t])
             sample_coordinates_d1 = np.einsum('ij,kj->ki', rotation_matrix, sample_coordinates_d1)
         else:
-            sample_coordinates_d1 = np.vstack((rotation_matrix[:, 0], np.diff(sample_coordinates, axis=0)))
+            if path_derivative is None:
+                sample_coordinates_d1 = np.vstack((rotation_matrix[:, 0], np.diff(sample_coordinates, axis=0)))
+            else:
+                sample_coordinates_d1 = np.array(path_derivative)
+                sample_coordinates_d1 = np.einsum('ij,kj->ki', rotation_matrix, sample_coordinates_d1)
 
         sample_coordinates_d1_norm = np.linalg.norm(sample_coordinates_d1, axis=1)
         sample_coordinates_d1_normed = sample_coordinates_d1 / sample_coordinates_d1_norm[:, None]
@@ -256,11 +258,13 @@ class Waveguide(object):
             else:
                 sample_width = np.array([width(x) for x in sample_t])
         else:
-            sample_width = np.array([(width if width else self._current_port.width), ])
+            if width is None:
+                sample_width = np.atleast_1d(self._current_port.width)
+            else:
+                sample_width = np.atleast_1d(width)
 
         if sample_width.ndim == 1:
             sample_width = sample_width[..., None]
-
         # Now we have everything to calculate the polygon
         polygons = []
         half_width = np.sum(sample_width, axis=-1) / 2
@@ -357,7 +361,7 @@ class Waveguide(object):
         self.add_bezier_to(port.origin, port.inverted_direction.angle, bend_strength, width, **kwargs)
         return self
 
-    def add_route_single_circle_to(self, final_coordinates, final_angle, max_bend_strength=None,
+    def add_route_single_circle_to(self, final_coordinates, final_angle, final_width=None, max_bend_strength=None,
                                    on_line_only=False):
 
         """
@@ -376,12 +380,13 @@ class Waveguide(object):
 
         :param final_coordinates: Final destination point.
         :param final_angle: Final angle of the waveguide.
+        :param final_width: Final width of the waveguide.
         :param max_bend_strength: The maximum allowed bending radius.
         :param on_line_only: Omit the last straight line and only route to described line.
         """
         # We are given an out angle, but the internal math is for inward pointing lines
         final_angle = normalize_phase(final_angle) + np.pi
-
+        final_width = final_width if final_width is not None else self.width
         # We need to to some linear algebra. We first find the intersection of the two waveguides
         r1 = self._current_port.origin
         r2 = np.array(final_coordinates)
@@ -407,19 +412,24 @@ class Waveguide(object):
 
         radius = min([max_bend_strength, max_poss_radius]) if max_bend_strength is not None else max_poss_radius
         d = abs(radius * np.tan(diff_angle / 2))
-
+        if on_line_only:
+            segments = [distance[0] - d, radius * diff_angle]
+        else:
+            segments = [distance[0] - d, radius * diff_angle, distance[1] - d]
+        segment_ratio = np.cumsum(segments / sum(segments))
+        segment_widths = [(final_width - self.current_port.width) * ratio + self.current_port.width for ratio
+                          in segment_ratio]
         tmp_wg = Waveguide.make_at_port(self._current_port)
-        tmp_wg.add_straight_segment(distance[0] - d)
-        tmp_wg.add_bend(-diff_angle, radius)
+        tmp_wg.add_straight_segment(length=distance[0] - d, final_width=segment_widths[0])
+        tmp_wg.add_bend(-diff_angle, radius, final_width=segment_widths[1])
+        if not on_line_only:
+            tmp_wg.add_straight_segment(distance[1] - d, final_width=segment_widths[2])
 
         self._segments.append(
             (self._current_port.copy(), tmp_wg.get_shapely_object(), tmp_wg.get_shapely_outline(), tmp_wg.length,
              tmp_wg.center_coordinates))
         self._current_port = tmp_wg.current_port
 
-        if not on_line_only:
-            self.add_straight_segment(distance[1] - d)
-
         return self
 
     def add_route_single_circle_to_port(self, port, max_bend_strength=None, on_line_only=False):
@@ -431,7 +441,8 @@ class Waveguide(object):
         :param max_bend_strength: The maximum allowed bending radius.
         :param on_line_only: Omit the last straight line and only route to line described by port.
         """
-        self.add_route_single_circle_to(port.origin, port.inverted_direction.angle, max_bend_strength, on_line_only)
+        self.add_route_single_circle_to(port.origin, port.inverted_direction.angle, port.width, max_bend_strength,
+                                        on_line_only)
         return self
 
     def add_straight_segment_to_intersection(self, line_origin, line_angle, **line_kw):
